/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */


#include "mygl.h"
#include "Background.h"
#include "Textures.h"

#include "setup.h"


static bool BackgroundActive;

#ifdef USEMOD_BACKGROUND
static GLint DisplayList = 0;
static Texture *T1 = NULL;
static Texture *T2 = NULL;
static Texture *T3 = NULL;
static Texture *T4 = NULL;
static Texture *T5 = NULL;
static Texture *T6 = NULL;
#endif


int BackgroundInit(void)
{
#ifdef USEMOD_BACKGROUND
	BackgroundActive = MaxTextureSize >= 512;
#else
	BackgroundActive = false;
#endif

#ifdef USEMOD_BACKGROUND
	if (BackgroundActive)
	{
		DisplayList = glGenLists(1);
		if (DisplayList == 0)  return 0;

#define TexWork(T,S)  \
{  \
	(T) = TextureLoadFromFile(S);  \
	if ((T) == NULL)  return 0;  \
}
		TexWork(T1,"Scene\\Bg00.tga");
		TexWork(T2,"Scene\\Bg06.tga");
		TexWork(T3,"Scene\\Bg12.tga");
		TexWork(T4,"Scene\\Bg18.tga");
		TexWork(T5,"Scene\\BgN0.tga");
		TexWork(T6,"Scene\\BgS0.tga");
#undef TexWork

		glNewList(DisplayList, GL_COMPILE);
		glDepthFunc(GL_ALWAYS);
		glColor3f(0,0,0);
		glEnable(GL_TEXTURE_2D);
#define A 1000

		glBindTexture(GL_TEXTURE_2D, T6->TexName());
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2i(0,0); glVertex3f(-A,-A,-A);  glTexCoord2i(1,0); glVertex3f( A,-A,-A);
		glTexCoord2i(0,1); glVertex3f(-A, A,-A);  glTexCoord2i(1,1); glVertex3f( A, A,-A);
		glEnd();

		glBindTexture(GL_TEXTURE_2D, T5->TexName());
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2i(0,1); glVertex3f(-A,-A, A);  glTexCoord2i(0,0); glVertex3f(-A, A, A);
		glTexCoord2i(1,1); glVertex3f( A,-A, A);  glTexCoord2i(1,0); glVertex3f( A, A, A);
		glEnd();

		glBindTexture(GL_TEXTURE_2D, T3->TexName());
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2i(1,0); glVertex3f(-A,-A,-A);  glTexCoord2i(1,1); glVertex3f(-A,-A, A);
		glTexCoord2i(0,0); glVertex3f( A,-A,-A);  glTexCoord2i(0,1); glVertex3f( A,-A, A);
		glEnd();

		glBindTexture(GL_TEXTURE_2D, T1->TexName());
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2i(0,0); glVertex3f(-A, A,-A);  glTexCoord2i(1,0); glVertex3f( A, A,-A);
		glTexCoord2i(0,1); glVertex3f(-A, A, A);  glTexCoord2i(1,1); glVertex3f( A, A, A);
		glEnd();

		glBindTexture(GL_TEXTURE_2D, T2->TexName());
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2i(1,0); glVertex3f(-A,-A,-A);  glTexCoord2i(1,1); glVertex3f(-A, A,-A);
		glTexCoord2i(0,0); glVertex3f(-A,-A, A);  glTexCoord2i(0,1); glVertex3f(-A, A, A);
		glEnd();

		glBindTexture(GL_TEXTURE_2D, T4->TexName());
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2i(0,0); glVertex3f( A,-A,-A);  glTexCoord2i(1,0); glVertex3f( A,-A, A);
		glTexCoord2i(0,1); glVertex3f( A, A,-A);  glTexCoord2i(1,1); glVertex3f( A, A, A);
		glEnd();

#undef A
		glDepthFunc(GL_LESS);
		glEndList();
	}
#endif

	return 1;
}


void BackgroundDone(void)
{
#ifdef USEMOD_BACKGROUND
#define TexWork(T)  \
{  \
	TextureDelete(T);  \
}
	if (BackgroundActive)
	{
		TexWork(T1);
		TexWork(T2);
		TexWork(T3);
		TexWork(T4);
		TexWork(T5);
		TexWork(T6);
#undef TexWork

		glDeleteLists(DisplayList, 1);
	}
#endif
}


void BackgroundRender(void)
{
#ifdef USEMOD_BACKGROUND
	if (BackgroundActive)
		glCallList(DisplayList);
	else
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
#else
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
#endif
}
