/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */


#include <stdio.h>
#include <assert.h>
#include "mygl.h"
#include "Cockpit.h"
#include "gui.h"
#include "Timers.h"
#include "Fonts.h"
#include "Input.h"]
#include "Log.h"
#include "CockpitMap.h"

#include "setup.h"


#define FRAME_RATE_FORMAT "FPS: %6.2f, peak: %6.2f, worst: %6.2f"
#define FRAME_RATE_X 0
#define FRAME_RATE_Y (guiInfo.winHeight - 30)
static char FRAME_RATE_UNKNOWN[] = "Meassuring ...";
static GLubyte FrameRateColor[3] = {255,255,255};

static GLuint dlFrameRate = 0;

static void CockpitEnter(void);
static void CockpitLeave(void);
static void FrameRateDraw(void);


int CockpitInit(void)
{
#ifdef USEMOD_COCKPIT
	// frame rate init
	dlFrameRate = glGenLists(1);
	if (dlFrameRate == 0) return 0;
	// CockpitMap init
	if (!CockpitMapInit()) { return 0; }
#endif
	return 1;
}


void CockpitDone(void)
{
#ifdef USEMOD_COCKPIT
	if (dlFrameRate != 0)
		glDeleteLists(dlFrameRate, 1);
#endif
}


void CockpitDraw(void)
{
#ifdef USEMOD_COCKPIT
	CockpitEnter();
	FrameRateDraw();
	CockpitMapDraw();
	CockpitLeave();
#endif
}


static inline GLenum GetMatrixMode(void)
{
	GLint mode;
	glGetIntegerv(GL_MATRIX_MODE, &mode);
	return mode;
}


static void CockpitEnter(void)
{
	logInfo(modCockpit, "Entering cockpit...");
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	// modelview matrix
	assert(GetMatrixMode() == GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	// projection matrix
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0.0, guiInfo.winWidth-1, 0.0, guiInfo.winHeight-1, -1.0, 1.0);
	// matrix mode for cockpit
	glMatrixMode(GL_MODELVIEW);
}


static void CockpitLeave(void)
{
	logInfo(modCockpit, "Leaving cockpit...");
	// projection matrix
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	// modelview matrix
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	// restore application's default state
	glEnable(GL_DEPTH_TEST);
	//glEnable(GL_LIGHTING);
}


static void FrameRateDraw(void)
{
	static bool Draw = true;
	static bool IsHold = false;

	if (InputState.FPSKey != IsHold)
	{
		if (InputState.FPSKey)
			Draw = !Draw;
		IsHold = InputState.FPSKey;
	}

	if (Draw)
	{
		if (!frIsFrameRateUpdated())
		{
			glCallList(dlFrameRate);
		}
		else
			if (frIsFrameRateValid())
			{
				char S[100];
				float AvgFR   = frGetAvgFrameRate();
				float PeakFR  = frGetPeakFrameRate();
				float WorstFR = frGetWorstFrameRate();
				sprintf(S, FRAME_RATE_FORMAT, AvgFR, PeakFR, WorstFR);

				glNewList(dlFrameRate, GL_COMPILE_AND_EXECUTE);
				font_Fotinos19->DrawString(&S[0], &FrameRateColor[0], (GLfloat)FRAME_RATE_X, (GLfloat)FRAME_RATE_Y);
				glEndList();
			}
			else
			{
				glNewList(dlFrameRate, GL_COMPILE_AND_EXECUTE);
				font_Fotinos19->DrawString(&FRAME_RATE_UNKNOWN[0], &FrameRateColor[0], (GLfloat)FRAME_RATE_X, (GLfloat)FRAME_RATE_Y);
				glEndList();
			}
	}
}






