/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */


#define WANT_MEMSET
#include "porting.h"

#include "Timers.h"
#include "Log.h"
#include "InputDevices.h"
#include "Input.h"

#include "setup.h"

#ifdef USE_SDL
# include "SDL_keysym.h"
#endif
#ifdef USE_GLUT
# define GLUT_API_VERSION 4
# define GLUT_BUILDING_LIB
# include <GL/glut.h>
#endif



InputStateTable InputState;
InputAssignmentsTable InputAssignments;

static void LoadAssignmentsDefaults(void);


timType LastUpdateTime = 0;
float MouseSensitivityHorizontal = (float)1.0/50;  // turn (in 1/rad) at 1 second when mouse is moved 10 points right
float MouseSensitivityVertical = (float)1.0/50;  // turn (in 1/rad) at 1 second when mouse is moved 10 points up
float KeyboardAssignedSpeed = 10;


static timType LastTickTime;
int InputUpdate(void)
{
#ifdef USEMOD_INPUT
	int r;
	int x,y;
//  timType c = timGetTicks();
//	double dt = timTicksToSec(c - LastTickTime);
//	LastTickTime = c;
	#define dt frLastFrameTime


	logInfo(modInput, "Getting status from input devices");


	r = MouseGetData(x,y);

	if (r)
	{
		InputState.RotLR = (float)x * MouseSensitivityHorizontal;
		InputState.RotDU = (float)y * MouseSensitivityVertical;
	} else {
		InputState.RotLR = 0;
		InputState.RotDU = 0;
		logWarning(modInput, "Can't read mouse state.");
	}


	if (!KeyboardUpdate())
	{
		r = 0;
		InputState.MovForward = 0;
		InputState.MovLR = 0;
//		memset(&InputState, 0, sizeof(InputState));
		logWarning(modInput, "Can't read keyboard state.");
	}
	else
	{
		InputState.Terminate = InputAssignments.Terminate.IsPressed();
		InputState.ChangeCamera = InputAssignments.ChangeCamera.IsPressed();
		InputState.FPSKey = InputAssignments.FPSKey.IsPressed();

		if (InputAssignments.MovL.IsPressed())
			if (InputAssignments.MovR.IsPressed())
				InputState.MovLR = 0;
			else
				InputState.MovLR = -KeyboardAssignedSpeed * (float)frLastFrameTime;
		else
			if (InputAssignments.MovR.IsPressed())
				InputState.MovLR = +KeyboardAssignedSpeed * (float)frLastFrameTime;
			else
				InputState.MovLR = 0;

		if (InputAssignments.MovB.IsPressed())
			if (InputAssignments.MovF.IsPressed())
				InputState.MovForward = 0;
			else
				InputState.MovForward = -KeyboardAssignedSpeed * (float)frLastFrameTime;
		else
			if (InputAssignments.MovF.IsPressed())
				InputState.MovForward = +KeyboardAssignedSpeed * (float)frLastFrameTime;
			else
				InputState.MovForward = 0;
	}

	return r;
#else
	return 1;
#endif
}


int InputInit(void)
{
#ifdef USEMOD_INPUT
	int r = 1;
	logInfo(modInput, "Starting devices initialization.");

	memset(&InputState, 0, sizeof(InputState));
	LoadAssignmentsDefaults();

	if (!MouseInit())
	{
		logError(modInput, "Can't initialize mouse.");
		r = 0;
	}
	if (!KeyboardInit())
	{
		logError(modInput, "Can't initialize keyboard.");
		r = 0;
	}

	logInfo(modInput, "Device initialization finished.");
	return r;
#else
	return 1;
#endif
}


void InputDone(void)
{
#ifdef USEMOD_INPUT
	logInfo(modInput, "Starting devices shooting down.");
	MouseDone();
	KeyboardDone();
	logInfo(modInput, "All devices terminated successfully.");
#endif
}


void InputAcquire(void)
{
#ifdef USEMOD_INPUT
	logInfo(modInput, "Acquiring devices.");
	if (!KeyboardAcquire())
		logWarning(modInput, "Can't acquire keyboard device.");
	if (!MouseAcquire())
		logWarning(modInput, "Can't acquire mouse device.");
	LastTickTime = timGetTicks();
#endif
}


void InputUnacquire(void)
{
#ifdef USEMOD_INPUT
	logInfo(modInput, "Unacquiring devices.");
	KeyboardUnacquire();
	MouseUnacquire();
#endif
}


void InputWindowCallback(void)
{
#ifdef USEMOD_INPUT
	logInfo(modInput, "Window modified callback.");
	if (!MouseWindowCallback())
		logError(modInput, "Can't reacquire mouse to modified window.");
#endif
}




bool InputAssignment::IsPressed(void)
{
	if (A[0] != 0)
	{
		if (KeyPressed(A[0]))
			return true;
		else
			if (A[1] != 0)
			{
				if (KeyPressed(A[1]))
					return true;
				else
					if (A[2] != 0)
					{
						if (KeyPressed(A[2]))
							return true;
						else
							if (A[3] != 0)
							{
								if (KeyPressed(A[3]))
									return true;
								else
									return false;
							}
							else return false;
					}
					else return false;
			}
			else return false;
	}
	else return false;
}


static void LoadAssignmentsDefaults(void)
{
	memset(&InputAssignments, 0, sizeof(InputAssignments));
#ifdef USE_SDL
	InputAssignments.MovF.AssignKeys(SDLK_UP,0,0,0);
	InputAssignments.MovB.AssignKeys(SDLK_DOWN,0,0,0);
	InputAssignments.MovR.AssignKeys(SDLK_RIGHT,0,0,0);
	InputAssignments.MovL.AssignKeys(SDLK_LEFT,0,0,0);
	InputAssignments.Terminate.AssignKeys(SDLK_ESCAPE,0,0,0);
	InputAssignments.ChangeCamera.AssignKeys(SDLK_SPACE,0,0,0);
	InputAssignments.FPSKey.AssignKeys(SDLK_F1,0,0,0);
#endif
#ifdef USE_GLUT
	InputAssignments.MovF.AssignKeys(GLUT_KEY_UP,0,0,0);
	InputAssignments.MovB.AssignKeys(GLUT_KEY_DOWN,0,0,0);
	InputAssignments.MovR.AssignKeys(GLUT_KEY_RIGHT,0,0,0);
	InputAssignments.MovL.AssignKeys(GLUT_KEY_LEFT,0,0,0);
	InputAssignments.ChangeCamera.AssignKeys(GLUT_SPACE,0,0,0);
	InputAssignments.FPSKey.AssignKeys(GLUT_F1,0,0,0);
#endif
}



