/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */

#ifndef _Input_h_
#define _Input_h_


int InputInit(void);
void InputDone(void);
int InputUpdate(void);

void InputAcquire(void);
void InputUnacquire(void);
void InputWindowCallback(void);


// InputStateTable - variables of user input devices
typedef struct {
	float RotLR; // rotation left-right - axes y
	float RotDU; // rotation down-up - axes x
	float Roll;  // rotation rolling - axes z
	float MovForward; // movement forward - axes z
	float MovLR; // movement left-right - axes x
	float MovDU; // movement down-up - axes y
	bool Terminate; // terminate application

	bool ChangeCamera;
//	bool ArrowUp; // for menu
//	bool ArrowDown; // for menu
	bool FPSKey;
} InputStateTable;


// InputAssignment - key assignment to one function
// four keys can be assigned to one function
#define AssignmentsCount 4
typedef struct {
	int A[AssignmentsCount];
	inline void AssignKeys(int C, int D, int E, int F)  { A[0]=C; A[1]=D; A[2]=E; A[3]=F; };
	bool IsPressed(void);
} InputAssignment;


// InputAssignmentsTable - table for keyboard key assignment
typedef struct {
	InputAssignment RotL;
	InputAssignment RotR;
	InputAssignment RotD;
	InputAssignment RotU;
	InputAssignment RolL;
	InputAssignment RolR;
	InputAssignment MovF;
	InputAssignment MovB;
	InputAssignment MovR;
	InputAssignment MovL;
	InputAssignment MovU;
	InputAssignment MovD;
	InputAssignment Terminate;

	InputAssignment ChangeCamera;
	InputAssignment FPSKey;
} InputAssignmentsTable;


extern InputStateTable InputState;
extern InputAssignmentsTable InputAssignments;

#endif /* _Input_h_ */
