/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */


//#ifndef VISUAL_CPP
#include <new.h>
//#endif
#define WANT_NOTHROW
#define WANT_DISABLE_MATH_EXCEPTIONS
#define WANT_SQR
#include "porting.h"
#include "SceneClasses.h"
#include "Scene.h"
#include "3dsLoad.h"
#include "gui.h"
#include "dlist.h"
#include "CockpitMap.h"
#include "RocketSmoke.h"
#include "Cameras.h"
#include "Background.h"

#include "setup.h"



char ScenePath[] = "scene\\";
FreezeMode FreezeStatus = fsPaused;
int StartCounter;
#define StartFramesCount 3


// functions in this module
int SceneInit(void);
void SceneDone(void);
void SceneTimeTick(const float DTime);
void SceneRender(void);
void SceneClear(void);
int SceneLoadDefault(void);
static int MakeDefaultScene(void);
static int MakeSimpleModel(void);
FreezeMode SceneFreezeStatus(void);
void SceneFreeze(bool F);
void Collision(void);


// StaticList - double-linked list of static scene objects
XList<scStatic> StaticList;
// MobileList - double-linked list of mobile scene objects
XList<scMobile> MobileList;
// BlendList - double-linked list of scene objects using blending
XList<scObject> BlendList;



int SceneInit(void)
{
#ifdef USEMOD_SCENE
	ThisPlayer = new(nothrow) scMobile;
	if (ThisPlayer == NULL) return 0;
#endif

	if (!SceneLoadDefault()) return 0;

		MainCamera.Set(ThisPlayer);
	return 1;
}


void SceneDone(void)
{
#ifdef USEMOD_SCENE
	delete ThisPlayer;
#endif
}


static void DoTimeTick(const float DTime)
{
// static objects don't need to call TimeTick()
//	scStatic *S = StaticList.First;
//	while (S!=NULL) {
//		S->TimeTick(DTime);
//		S = (scStatic*)S->Succ;
//	}

	scMobile *M = MobileList.First;
	while (M!=NULL) {
		M->TimeTick(DTime);
		M = (scMobile*)M->Next;
	}

	scObject *O = BlendList.First;
	while (O!=NULL) {
		O->TimeTick(DTime);
		O = (scStatic*)O->Next;
	}

	Collision();
}


void SceneTimeTick(const float DTime)
{
	if (FreezeStatus == fsRunning)  DoTimeTick(DTime);
	else
		if (FreezeStatus == fsStarting)
		{
			StartCounter++;
			if (StartCounter == StartFramesCount)
			{
				FreezeStatus = fsRunning;
				DoTimeTick(DTime);
			}
			else
				DoTimeTick(0);
		}
		else
			DoTimeTick(0);
}


void SceneRender(void)
{
#ifndef USEMOD_SCENE
	glLoadIdentity();
#endif

	// Borland needs disable math exception during rendering through OpenGL
	Disable8087Exceptions();

	// scene rendering ...
	BackgroundRender();

	scStatic *S = StaticList.First;
	while (S!=NULL) {
		S->Render();
		S = (scStatic*)S->Next;
	}

	scMobile *M = MobileList.First;
	while (M!=NULL) {
		M->Render();
		M = (scMobile*)M->Next;
	}

	scObject *O = BlendList.First;
	while (O!=NULL) {
		O->Render();
		O = (scObject*)O->Next;
	}

	// enable math exceptions for Borland
	Enable8087Exceptions();
}


void SceneClear(void)
{
	scStatic *S = StaticList.First;
	scStatic *TS;
	while (S!=NULL) {
		TS = (scStatic*)S->Next;
		delete S;
		S = TS;
	}
	StaticList.First = NULL;
	StaticList.Last = NULL;

	scMobile *M = MobileList.First;
	scMobile *TM;
	while (M!=NULL) {
		TM = (scMobile*)M->Next;
		delete M;
		M = TM;
	}
	MobileList.First = NULL;
	MobileList.Last = NULL;
}


int SceneLoadDefault(void)
{
	SceneClear();
#ifdef USEMOD_SCENE
	MatrixLoadIdentity(&ThisPlayer->Matrix);
#endif
	return MakeDefaultScene();
}


static int MakeDefaultScene(void)
{
#ifdef USEMOD_SCENE
	int DL;
	if ((DL = Load3DS("Cone_3.3ds")) > 0)
	{
		scStatic *S = new scStatic;
		S->DisplayList = DL;
		S->ColRadius = 20;
		StaticList.Insert(S);
	}
	if ((DL = Load3DS("Fighter1.3ds")) > 0)
	{
		scRocket *R = new scRocket;
		R->DisplayList = DL;
		R->ColRadius = 20;
		R->SetPosition(20,-10,-35);
		R->SetSpeed(0,0,-5);
		MobileList.Insert(R);

		RocketSmoke *Smoke = new RocketSmoke;
		if (Smoke != NULL)
		{
			BlendList.Insert(Smoke);
			Smoke->Assign(R, 30);
		}
	}
	if ((DL = Load3DS("Fighter2.3ds")) > 0)
	{
		scRocket *R = new scRocket;
		R->DisplayList = DL;
		R->ColRadius = 20;
		R->SetPosition(20,-10,-150);
		//R->Rotate(1,0,0);
		MobileList.Insert(R);
	}
	if ((DL = Load3DS("Imperial.3ds")) > 0)
	{
		scMobile *M = new scMobile;
		M->DisplayList = DL;
		M->SetPosition(0,10,-100);
		M->ColRadius = 20;
		MobileList.Insert(M);
	}

	//vezme si ukazatele na seznam objektu pro CockpitMap TODO
	SetShortcut2SceneObjs(MobileList.First, StaticList.First);

#else
	MakeSimpleModel();
#endif
	return 1;
}


#ifdef USEMOD_COLLISIONS
static int srazka(scStatic *A, scStatic *B)
{
		GLfloat suma=0.0;
		GLfloat vectorAB[3]={A->GetPosX() - B->GetPosX(),
							 A->GetPosY() - B->GetPosY(),
							 A->GetPosZ() - B->GetPosZ()};
		for (int i=0; i<3; i++) {
				suma += sqr(vectorAB[i]);
		}
		return (suma <= sqr(A->ColRadius + B->ColRadius));
}
#endif

void Collision(void)
{
#ifdef USEMOD_COLLISIONS
	scStatic *S = StaticList.First;
	scMobile *M = MobileList.First;
	while (M!=NULL) {
		scMobile *TM = MobileList.First;
		while (TM!=NULL) {
			if (TM!=M) { // at neporavnavam stejny objekt
				if (srazka(M, TM) == 1) {
					M->SetSpeed(0,0,0);
				}
			}
			TM = (scMobile*)TM->Next;
		}
		while (S!=NULL) {
			if (srazka(M, S) == 1) {
				M->SetSpeed(0,0,0);
			}
			S = (scStatic*)S->Next;
		}
		M = (scMobile*)M->Next;
	}
#endif
}


static int MakeSimpleModel(void)
{
#define D 5.0
#define S 2.0
	int DL = glGenLists(1);
	glNewList(DL, GL_COMPILE);
		glBegin(GL_POLYGON);
		glColor3f(0.0,1.0,0.0);  glVertex3f(-S,-S,-D);
		glColor3f(0.0,0.0,1.0);  glVertex3f(+S,-S,-D);
		glColor3f(1.0,0.0,0.0);  glVertex3f(+S,+S,-D);
		glColor3f(1.0,1.0,0.0);  glVertex3f(-S,+S,-D);
		glEnd();
	glEndList();
#undef S
#undef D

	scMobile *M = new scMobile;
	M->DisplayList = DL;
	MobileList.Insert(M);

	return guiCheckForGLErrors(modScene, "Can't compile default scene to OpenGL (cube model), error(s): %s");
}



FreezeMode SceneFreezeStatus(void)
{
	return FreezeStatus;
}

void SceneFreeze(bool F)
{
	if (F)
	{
		if (FreezeStatus != fsPaused)
		{
			FreezeStatus = fsPaused;
		}
	}
	else
	{
		if (FreezeStatus == fsPaused)
		{
			FreezeStatus = fsStarting;
			StartCounter = 0;
		}
	}
}

