/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */


#ifdef WINDOZE
#include <windows.h>
#include <stdio.h>
#include <assert.h>
#else
#include <time.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/times.h>
#endif
#include <limits.h>
#pragma hdrstop // for C++ Builder compiler speed up

#include "Timers.h"
#include "Log.h"



#ifdef WINDOZE
	// need for max for 64bit integer
	#ifdef BORLANDLIB
	// Borland libraries defines this
	#define timTypeMax _I64_MAX
	#else
		#ifdef VISUAL_CPP
		// Visual Studio 64bit integer type
		#define timTypeMax _I64_MAX
		#else
		// Mingw32 libraries has this
		#define timTypeMax LONG_LONG_MAX
		#endif
	#endif
#else
#define timTypeMax LONG_MAX
#endif

double timTickTime;
timType frLastFrameTicks;
double frLastFrameTime;


int timInit(void)
{
#ifdef WINDOZE
	LARGE_INTEGER Frequency;
	if (!QueryPerformanceFrequency(&Frequency))
	{
		logFatalError(modTimers, "High resolution timer not supported by hardware.");
		return 0;
	}
	timTickTime = 1.0/(long double)Frequency.QuadPart;
#else
	timTickTime = 1/(double)/*CLK_TCK*/100;
#endif
	return 1;
}


timType timGetTicks(void)
{
#ifdef WINDOZE
  LARGE_INTEGER Counter;
  if (!QueryPerformanceCounter(&Counter))
    {
      logFatalError(modTimers, "Can't read high resolution timer.");
      return 0;
    }
  return Counter.QuadPart;
#else
  return times(NULL);
#endif
}


double timGetRT(void)
{
#ifdef WINDOZE
  LARGE_INTEGER Counter;
  assert(QueryPerformanceCounter(&Counter));
  return (long)Counter.QuadPart * timTickTime;
#else
  return times(NULL) * timTickTime;
#endif
}




double FramesTicks;
int FramesCount;
timType LastFrame;
bool IsFrameRateUpdated;
bool IsFrameRateValid;
float TicksToRefresh;
timType MinTicks;
timType MaxTicks;

float AvgFR;
float PeakFR;
float WorstFR;

float frGetAvgFrameRate(void)  { return AvgFR; }
float frGetPeakFrameRate(void)  { return PeakFR; }
float frGetWorstFrameRate(void)  { return WorstFR; }
bool frIsFrameRateUpdated(void)  { return IsFrameRateUpdated; }
bool frIsFrameRateValid(void)  { return IsFrameRateValid; }


int frInit(void)
{
	frReset();
	TicksToRefresh = (float)0.5 / (float)timTickTime;
	return 1;
}


void frReset(void)
{
	FramesTicks = 0;
	FramesCount = 1;
	MinTicks = timTypeMax;
	MaxTicks = 0;
	LastFrame = timGetTicks();
	IsFrameRateUpdated = true;
	IsFrameRateValid = false;
	AvgFR = -1.0;
	PeakFR = -1.0;
	WorstFR = -1.0;
}


void frNextFrame(void)
{
	timType time = timGetTicks();
	frLastFrameTicks = time - LastFrame;
	frLastFrameTime = timTicksToSec(frLastFrameTicks);
	LastFrame = time;
	FramesTicks += frLastFrameTicks;

	if (frLastFrameTicks > MaxTicks)
		MaxTicks = frLastFrameTicks;
	if (frLastFrameTicks < MinTicks)
		MinTicks = frLastFrameTicks;

	if (FramesTicks < TicksToRefresh)
	{
		IsFrameRateUpdated = false;
		++FramesCount;
	}
	else
	{
		IsFrameRateUpdated = true;
		IsFrameRateValid = true;
		AvgFR = (float)FramesCount / ((float)FramesTicks * (float)timTickTime);
		PeakFR = (float)1.0 / ((float)MinTicks * (float)timTickTime);
		WorstFR = (float)1.0 / ((float)MaxTicks * (float)timTickTime);
		FramesTicks = 0;
		FramesCount = 1;
		MinTicks = timTypeMax;
		MaxTicks = 0;
	}
}


