/*
 * The POG project
 * Copyright 2000 The POG project team
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 */

#include "gui.h"
#include "Input.h"
#include "SDL.h"

#include "setup.h"


#ifdef USE_SDL


#pragma comment (lib, "sdl.lib")

int guiSDLInit(void);
void guiSDLDone(void);
void guiSdlMainLoop(void);



int guiSdlInit(void)
{
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
	{
		logError(modGUI, "Unable to initialize SDL: %s.", SDL_GetError());
		return 0;
	}

	/* Set the title bar in environments that support it */
	SDL_WM_SetCaption("VRapps", NULL);

	/* Create a 640x480 OpenGL screen */
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	if ( SDL_SetVideoMode(guiInfo.winWidth, guiInfo.winHeight, 16, SDL_OPENGL) == NULL ) {
		logError(modGUI, "Unable to create OpenGL screen: %s.", SDL_GetError());
		return 0;
	}

    guiWindowReshape(guiInfo.winWidth, guiInfo.winHeight);

    return 1;
}


void guiSdlDone(void)
{
	SDL_Quit();
}


void guiSdlMainLoop()
{
	SDL_EventState(SDL_KEYDOWN, SDL_IGNORE);
	SDL_EventState(SDL_KEYUP, SDL_IGNORE);
    SDL_PumpEvents();
	while (!InputState.Terminate) {

		guiRenderScene();
		SDL_GL_SwapBuffers();
		guiAfterRendering();

		SDL_Event event;
		while ( SDL_PollEvent(&event) ) {
			switch (event.type)
			{
				case SDL_VIDEORESIZE: guiWindowReshape(event.resize.w,event.resize.h); break;
				case SDL_QUIT: InputState.Terminate = 1; break;
			}
		}
	}
}


#endif /* USE_SDL */
